package jpel.bridge;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;
import java.util.Stack;

import jpel.language.Declaration;
import jpel.language.DeclarationException;
import jpel.language.DeclarationModule;
import jpel.language.DeclarationType;
import jpel.language.Environment;
import jpel.language.EnvironmentBuilder;
import jpel.language.EnvironmentFactory;
import jpel.language.EnvironmentFactoryException;
import jpel.language.ExpressionId;
import jpel.tree.Node;
import jpel.tree.NodeBuilder;
import jpel.tree.NodeConversor;
import jpel.tree.NodeException;
import jpel.tree.NodeProcessor;

/**
 * Define o processo de converso entre uma rvore e um ambiente da linguagem
 * funcional.
 */
public class NodeConversorEnvironment implements NodeConversor {

    private NodeProcessor processor;
    private EnvironmentFactory environmentFactory;
    private Environment environment;

    private Stack stack;

    public NodeConversorEnvironment() throws ClassNotFoundException,
        IllegalAccessException, InstantiationException {
        this(
            NodeBuilder.lookupNodeProcessor(Environment.class.getName()),
            EnvironmentBuilder.getEnvironmentFactory()
            );
    }

    public NodeConversorEnvironment(NodeProcessor processor,
        EnvironmentFactory environmentFactory) {
        this.processor = processor;
        this.environmentFactory = environmentFactory;
        this.environment = null;
        this.stack = new Stack();
    }

    public String[] getTypes() {
        return new String[] {
            Environment.class.getName()};
    }

    public String getDescription() {
        return "Environment conversor";
    }

    /**
         * Realiza a converso de uma estrutura hierrquica em um ambiente da linguagem
     * funcional.
     * @param node N raiz da rvore com as declaraes da linguagem funcional.
     * @return Um ambiente que corresponde ao n declarado.
     * @throws NodeException Quando h erros na converso do n.
     */
    public Object convert(Node node) throws NodeException {
        try {
            node = processor.process(node);
            environment = environmentFactory.produce();
            elabore(node, environment);
            environment.resolveRelatives();
            return environment;
        }
        catch(EnvironmentFactoryException exc) {
            throw new NodeException(exc.getMessage(), exc);
        }
    }

    private void elabore(Node node, Environment env) throws NodeException {
        try {
            if(node.isRoot()) {
                clear();
            }
            if(node.isFolder()) {
                if(!node.isRoot()) {
                    DeclarationModule dec = (DeclarationModule) node.getValue();
                    dec.setSubEnvironment(environmentFactory.empty());
                    dec.setModule(currentModule());
                    dec.execute(env);
                    push(dec.getName());
                }
                Iterator iter = node.children();
                while(iter.hasNext()) {
                    elabore((Node) iter.next(), env);
                }
                if(!node.isRoot()) {
                    pop();
                }
            }
            if(node.isLeaf()) {
                Object obj = node.getValue();
                if(obj instanceof Declaration) {
                    DeclarationType type = ((Declaration) obj).getType();
                    Declaration dec = (Declaration) obj;
                    dec.setModule(currentModule());
                    dec.execute(env);
                }
                else {
                    throw new NodeException("Object type " + obj + "."
                        + obj.getClass().getName() + " not compatible!");
                }
            }
        }
        catch(EnvironmentFactoryException exc) {
            throw new NodeException(exc.getMessage(), exc);
        }
        catch(DeclarationException exc) {
            throw new NodeException(exc.getMessage(), exc);
        }
    }

    private void clear() {
        stack.clear();
    }

    private void push(ExpressionId name) {
        if(stack.isEmpty()) {
            stack.push(name);
        }
        else {
            ExpressionId id = (ExpressionId) stack.peek();
            stack.push(new ExpressionId(id.getName()
                + ExpressionId.SEPARATOR + name.getName()));
        }
    }

    private void pop() {
        stack.pop();
    }

    private ExpressionId currentModule() {
        ExpressionId result = null;
        if(!stack.isEmpty()) {
            result = (ExpressionId) stack.peek();
        }
        return result;
    }
}
