package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

import jpel.util.Debugger;

/**
 * Implementao bsica de um ambiente da linguagem.
 */
public class EnvironmentImpl implements Environment {

    private Stack stack;
    private Context current;

    public EnvironmentImpl() {
        stack = new Stack();
        current = new Context();
        stack.push(current);
    }

    private class Context {
        public List list;
        public HashMap hash;

        public Context() {
            list = new LinkedList();
            hash = new HashMap();
        }

        public String toString() {
            return hash.toString();
        }
    }

    private class Entry {
        public Abstraction abstraction;
        public Environment subEnvironment;

        public Entry(Abstraction abstraction, Environment subEnvironment) {
            this.abstraction = abstraction;
            this.subEnvironment = subEnvironment;
        }

        public String toString() {
            return(abstraction != null ? abstraction + ""
                : "") + ":" + (subEnvironment != null ? subEnvironment + ""
                : "");
        }
    }

    public void bind(ExpressionId id, Abstraction abstraction) {
        Entry e = (Entry) current.hash.get(id);
        if(abstraction != null) {
            if(e == null) {
                bind(id, new Entry(abstraction, null));
            }
            else {
                e.abstraction = abstraction;
            }
        }
    }

    public void bind(ExpressionId id, Environment subEnvironment) {
        Entry e = (Entry) current.hash.get(id);
        if(subEnvironment != null) {
            if(e == null) {
                bind(id, new Entry(null, subEnvironment));
            }
            else {
                e.subEnvironment = subEnvironment;
            }
        }
    }

    private void bind(ExpressionId id, Entry e) {
        current.hash.put(id, e);
        if(!current.list.contains(id)) {
            current.list.add(id);
        }
    }

    public Abstraction lookup(ExpressionId id) throws IdNotFoundException {
        Entry e = privateLookup(id);
        if(e.abstraction == null) {
            throw new IdNotFoundException(id,
                "Identifier " + id + " not found.");
        }
        return e.abstraction;
    }

    public Environment lookupSub(ExpressionId id) throws IdNotFoundException {
        Entry e = privateLookup(id);
        if(e.subEnvironment == null) {
            throw new IdNotFoundException(id,
                "Environment " + id + " not found.");
        }
        return e.subEnvironment;
    }

    private Entry privateLookup(ExpressionId id) throws IdNotFoundException {
        EnvironmentImpl env = this;
        ExpressionId tmp = id;
        while(tmp.hasModule()) {
            String str = tmp.getName();
            int pos = str.indexOf(ExpressionId.SEPARATOR);
            ExpressionId part = new ExpressionId(str.substring(0, pos));
            env = (EnvironmentImpl) env.lookupSub(part);
            tmp = new ExpressionId(str.substring(pos + 1));
        }
        Entry result = null;
        Object[] all = env.stack.toArray();
        for(int i = all.length - 1; result == null && i >= 0; i--) {
            result = (Entry) (((Context) all[i]).hash).get(tmp);
        }
        if(result == null) {
            throw new IdNotFoundException(id, "Id '" + id + "' not found!");
        }
        return result;
    }

    public Iterator keys() {
        ExpressionList list = new ExpressionListImpl();
        Iterator iter = current.list.iterator();
        while(iter.hasNext()) {
            list.append((Expression) iter.next());
        }
        return list.iterator();
    }

    public void addContext() {
        current = new Context();
        stack.push(current);
        //Debugger.println("Environment","addContext",toString());
    }

    public void removeContext() {
        Object remove = stack.pop();
        current = (Context) stack.peek();
        //Debugger.println("Environment","removeContext","Removed '"+remove+"' from '"+toString());
    }

    public void resolveRelatives() {
        Iterator iter = keys();
        while(iter.hasNext()) {
            ExpressionId item = (ExpressionId) iter.next();
            resolveLocals(null, item, this);
        }
    }

    private void resolveLocals(String module, ExpressionId id, Environment env) {
        Abstraction abs = null;
        Environment sub = null;
        try {
            abs = env.lookup(id);
        }
        catch(IdNotFoundException ex) {}
        try {
            sub = env.lookupSub(id);
        }
        catch(IdNotFoundException ex) {}
        if(abs != null) {
            ExpressionList fArgs = new ExpressionListImpl();
            abs.getArguments().freeVariable(fArgs);
            ExpressionList fBody = new ExpressionListImpl();
            abs.getBody().freeVariable(fBody);
            fBody.remove(fArgs);
            Iterator iter = fBody.iterator();
            MapReplace replace = new MapReplaceImpl();
            while(iter.hasNext()) {
                ExpressionId item = (ExpressionId) iter.next();
                if(item.isRelative()) {
                    ExpressionId newId = new ExpressionId((module != null
                                         ? module : "") + item.getName());
                    replace.put(item, newId);
                    Debugger.println("Environment", "resolve",
                        "Relative:" + item + ", changed to " + newId);
                }
            }
            Abstraction newAbs = (Abstraction) abs.createClone(); // substitui a abstrao
            Expression newBody = newAbs.getBody(); // por uma cpia com os Ids
            newBody = newBody.rebuild(replace); // locais substitudos.
            newAbs.setBody(newBody);
            env.bind(id, newAbs);
        }
        if(sub != null) {
            String newModule = (module != null
                               ? module + ExpressionId.SEPARATOR
                               : "") + id.getName();
            Iterator iter = sub.keys();
            while(iter.hasNext()) {
                ExpressionId item = (ExpressionId) iter.next();
                resolveLocals(newModule, item, sub);
            }
        }
    }

    public Environment createClone() {
        EnvironmentImpl clone = new EnvironmentImpl();
        Iterator iter = keys();
        while(iter.hasNext()) {
            ExpressionId id = (ExpressionId) iter.next();
            Abstraction abs = null;
            try {
                abs = lookup(id);
            }
            catch(IdNotFoundException ex) {}
            Environment env = null;
            try {
                env = lookupSub(id);
            }
            catch(IdNotFoundException ex) {}

            ExpressionId newId = (ExpressionId) id.createClone();
            if(abs != null) {
                clone.bind(newId, (Abstraction) abs.createClone());
            }
            if(env != null) {
                clone.bind(newId, env.createClone());
            }
        }
        return clone;
    }

    public String toString() {
        StringBuffer str = new StringBuffer();
        str.append(stack.toString());
        return str.toString();
    }

    public static void main(String[] args) {
        try {
            Environment env = new EnvironmentImpl();

            ExpressionId a = new ExpressionId("a");
            Environment aEnv = new EnvironmentImpl();
            env.bind(a, aEnv);
            System.out.println("env=" + env);
            System.out.println("env(a)=" + env.lookup(a));

            ExpressionId b = new ExpressionId("b");
            Environment bEnv = new EnvironmentImpl();
            env.bind(b, bEnv);
            System.out.println("env=" + env);
            System.out.println("env(b)=" + env.lookup(b));

            ExpressionId c = new ExpressionId("c");
            Environment cEnv = new EnvironmentImpl();
            bEnv.bind(c, cEnv);
            cEnv.bind(new ExpressionId("x"), new Abstraction(null,
                ExpressionBoolean.getBoolean(true)));
            System.out.println("env=" + env);
            //System.out.println("env(a.b.c)=" + env.lookupSub(new ExpressionId("a.b.c")));
            System.out.println("lookup(b.c)="
                + env.lookup(new ExpressionId("b.c")));
            System.out.println("lookupSub(b.c)="
                + env.lookupSub(new ExpressionId("b.c")));
            System.out.println("lookup(b.c.x)="
                + env.lookup(new ExpressionId("b.c.x")));
            System.out.println("lookupSub(b.c.x)="
                + env.lookupSub(new ExpressionId("b.c.x")));
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }
    }
}
