package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.Serializable;

/**
 * Esta  a unidade bsica da linguagem funcional. Uma expresso,
 * que possui: Um tipo associado, uma lista de variveis
 * livres, um mecanismo de substituio destas variveis livres, uma forma
 * de ser avaliada e uma relao de equivalncia com as outras expresses.
 */
public interface Expression extends Serializable {

    /**
         * Retorna a fonte que gerou a expresso. Suponha que foi aplicada um parser
     * sobre a expresso:<br>
     *      x == 0    = 0<br>
     *      otherwise = x * fat (x-1)
     * A traduo interna desta funo no representa bem esta expresso, para
     * isso podemos ajustar esta fonte de dados para indicar que este texto
     * originou determinada expresso, o que facilita a construo de editores
     * grficos para a linguagem.
     * <p>
     * Na definio do toString(), quando este campo  diferente de null, ele
     * correponde a descrio do objeto, caso contrrio usa-se a implementao
     * do toString() especfico.
     * @return A fonte de dados que gerou a expresso.
     */
    Object getSource();

    /**
     * Ajusta a fonte de dados que gerou a expresso.
     * @param source A fonte dos dados.
     */
    void setSource(Object source);

    /**
     * Retorna o dado associado  expresso. Este recurso  necessrio para a
     * gerao de objetos Java que no fazem parte diretamente da linguagem,
     * como uma conexo ao banco de dados, ou mesmo os "wrapers" dos tipos
     * primitivos.
     * @return O objeto embutido dentro de uma expresso.
     */
    Object getData();

    /**
     * Retorna o dado associado  expresso.
     * @param type O tipo do objeto desejado.
     * @return O objeto embutido dentro de uma expresso se ele for do tipo
     * solicitado, null, caso contrrio.
     */
    Object getData(Class type);

    /**
     * Ajusta o dado agregado  expresso.
     * @param data O objeto que deve ser agregado a expresso.
     */
    void setData(Object data);

    /**
     * Fornece o tipo da expresso.
     * @return O tipo da expresso.
     */
    ExpressionType getType();

    /**
     * Ajusta o tipo da expresso.
     * @param newType Novo tipo da expresso.
     */
    void setType(ExpressionType newType);

    /**
     * Guarda as variveis livres da expresso em uma lista.
     * @param list A lista onde as variveis livres vo ser inseridas.
     */
    void freeVariable(ExpressionList list);

    /**
         * Permite que sub-expresses sejam substibudas por outras, de acordo com o
     * mapeamento dado.
     * @param map O mapeamento que guarda as substituies.
     * @return Uma nova expresso com as devidas substituies.
     */
    Expression rebuild(MapReplace map);

    /**
     * Executa uma expresso de acordo com o ambiente definido.
     * @param environment Ambiente no qual a expresso deve ser executada.
     * @return A expresso avaliada.
         * @throws IdNotFoundException Quando um identificador necessrio a execuo
     * da expresso no e encontrado no ambiente.
     * @throws ExecutionException Quando os tipos necessrios para a correta
     * avaliao da expresso no so respeitados.
     */
    Expression eval(Environment environment) throws ExecutionException;

    /**
     *  Indica se uma expresso pode ser considerada equivalente  outra.
     * @param expression Expresso ao qual se deseja verificar a equivalncia.
     * @return true, caso as expresses possam ser consideradas equivalentes,
     * false, caso contrrio.
     */
    boolean equivalent(Expression expression);

    /**
     * Retorna um clone da expresso.
     * @return O clone da expresso.
     */
    Expression createClone();
}