package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;

/**
 * Representa o tipo String da lisnguagem. Apesar de ser uma lista, apenas
 * expresses que sejam StringableExpression podem ser adicionadas, removidas,
 * procuradas ou comparadas.
 */
public class ExpressionString extends ExpressionObject implements
    ExpressionList, StringableExpression, ComparableExpression {

    private String value;

    public ExpressionString(boolean value) {
        this(String.valueOf(value));
    }

    public ExpressionString(char value) {
        this(String.valueOf(value));
    }

    public ExpressionString(int value) {
        this(String.valueOf(value));
    }

    public ExpressionString(String value) {
        super(ExpressionType.STRING, value);
        setExpression(value);
    }

    public void setExpression(String newExpression) {
        value = newExpression;
    }

    public String getExpression() {
        return value;
    }

    public ExpressionList clear() {
        setExpression("");
        return this;
    }

    public ExpressionList append(Expression expression) {
        value += ((StringableExpression) expression).asString();
        return this;
    }

    public ExpressionList append(ExpressionList expressionList) {
        Iterator i = expressionList.iterator();
        while(i.hasNext()) {
            append((Expression) i.next());
        }
        return this;
    }

    public ExpressionList remove(Expression expression) {
        value = replace(value,
                ((StringableExpression) expression).asString(), "");
        return this;
    }

    public ExpressionList remove(ExpressionList expressionList) {
        Iterator i = expressionList.iterator();
        while(i.hasNext()) {
            remove((Expression) i.next());
        }
        return this;
    }

    public boolean contains(Expression expression) {
        return(value.indexOf(((StringableExpression) expression).asString())
            >= 0);
    }

    public boolean contains(ExpressionList expressionList) {
        boolean result = true;
        Iterator i = expressionList.iterator();
        while(result && i.hasNext()) {
            result = contains((Expression) i.next());
        }
        return result;
    }

    public long length() {
        return value.length();
    }

    public boolean isEmpty() {
        return "".equals(value);
    }

    public Expression head() {
        if(isEmpty()) {
            return null;
        }
        return new ExpressionCharacter(value.charAt(0));
    }

    public ExpressionList tail() {
        if(isEmpty()) {
            return this;
        }
        return new ExpressionString(value.substring(1));
    }

    public Iterator iterator() {
        return new ExpressionListIterator(this);
    }

    public void freeVariable(ExpressionList list) { /* do nothing */}

    public Expression rebuild(MapReplace map) {
        return new ExpressionString(value);
    }

    public Expression eval(Environment environment) throws
        IdNotFoundException, BadTypedException {
        return this;
    }

    public boolean equivalent(Expression expression) {
        boolean result = false;
        if(expression instanceof ExpressionString) {
            result = value.equals(((ExpressionString) expression).value);
        }
        return result;
    }

    public int compareTo(ComparableExpression expression) throws
        BadTypedException {
        if(expression instanceof ExpressionList) {
            if(isEmpty() && ((ExpressionList) expression).isEmpty()) {
                return 0;
            }
        }
        if(!(expression instanceof ExpressionString)) {
            throw new BadTypedException(expression,
                this +" can not be compared to " + expression
                + ". Only Strings can be compared to Strings.");
        }
        return value.compareTo(((ExpressionString) expression).value);
    }

    public String asString() {
        return value;
    }

    public String toString() {
        return "\"" + value + "\"";
    }

    public String toString(String prefix, String separator, String suffix) {
        return prefix + value + suffix;
    }

    /**
     * Tranforma um Streing que contenha caracteres de scape em ums String
     * correposdente em Java.
     * @param str String que ser normalizada.
     * @return A String com os caracters de scape devidamente substitudos.
     */
    public static String normalize(String str) {
        String result = str;
        result = replace(result, "\\\\", "\\");
        result = replace(result, "\\\"", "\"");
        result = replace(result, "\\\'", "\'");
        result = replace(result, "\\n", "\n");
        result = replace(result, "\\r", "\r");
        result = replace(result, "\\t", "\t");
        return result;
    }

    private static String replace(String str, String pattern, String replace) {
        int s = 0;
        int e = 0;
        int pattern_length = pattern.length();
        StringBuffer result = new StringBuffer(str.length());
        while((e = str.indexOf(pattern, s)) >= 0) {
            result.append(str.substring(s, e));
            result.append(replace);
            s = e + pattern_length;
        }
        result.append(str.substring(s));
        return result.toString();
    }
}
