package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;

/**
 * Rerpesenta o operador de mltipla escolha.
 * <p>
 * Syntax:
 * <br>     | case condition1 = expression1;
 * <br>     | case condition2 = expression2;
 * <br>     ...
 * <br>     | case conditionN = expressionN;
 * <br>     | otherwise = defaultExpression;
 * <br> Exemplo:
 * <br> fat n =
 * <br>     | n == 1    = 0
 * <br>     | otherwise = n * fat (n - 1)
 */
public class ExpressionSwitch extends AbstractExpression {

    private ExpressionList conditions;
    private ExpressionList expressions;
    private Expression otherwise;

    public ExpressionSwitch(ExpressionList conditions,
        ExpressionList expressions, Expression otherwise) {
        super(ExpressionType.UNKNOWN);
        setConditions(conditions);
        setExpressions(expressions);
        setOtherwise(otherwise);
    }

    public ExpressionList getConditions() {
        return conditions;
    }

    public void setConditions(ExpressionList newConditions) {
        conditions = newConditions;
    }

    public void setExpressions(ExpressionList newExpressions) {
        expressions = newExpressions;
    }

    public ExpressionList getExpressions() {
        return expressions;
    }

    public void setOtherwise(Expression newOtherwise) {
        otherwise = newOtherwise;
    }

    public Expression getOtherwise() {
        return otherwise;
    }

    public void freeVariable(ExpressionList list) {
        Iterator i = conditions.iterator();
        Iterator j = expressions.iterator();
        while(i.hasNext() && j.hasNext()) {
            ((Expression) i.next()).freeVariable(list);
            ((Expression) j.next()).freeVariable(list);
        }
        otherwise.freeVariable(list);
    }

    public Expression rebuild(MapReplace map) {
        MapReplace novo = map.mirror();
        return new ExpressionSwitch((ExpressionList) getConditions().rebuild(
            novo), (ExpressionList) getExpressions().rebuild(novo),
            getOtherwise().rebuild(novo));
    }

    public Expression eval(Environment environment) throws
        ExecutionException {
        Expression result = null;
        Iterator i = conditions.iterator();
        Iterator j = expressions.iterator();
        Expression auxI;
        Expression auxJ;
        while(result == null && i.hasNext() && j.hasNext()) {
            auxI = (Expression) i.next();
            auxJ = (Expression) j.next();
            if(((ExpressionBoolean) auxI.eval(environment)).
                getBoolean()) {
                result = auxJ.eval(environment);
            }
        }
        if(result == null) {
            result = otherwise.eval(environment);
        }
        return result;
    }

    public boolean equivalent(Expression expression) {
        boolean result = false;
        if(expression instanceof ExpressionSwitch) {
            ExpressionSwitch e = (ExpressionSwitch) expression;
            result = conditions.equivalent(e.conditions)
                     && expressions.equivalent(e.expressions)
                     && otherwise.equivalent(e.otherwise);
        }
        return result;
    }

    public String toString() {
        if(getSource() != null) {
            return String.valueOf(getSource());
        }
        String str = null;
        Iterator i = conditions.iterator();
        Iterator j = expressions.iterator();
        while(i.hasNext() && j.hasNext()) {
            str = (str == null ? "| "
                  : str + " |") + i.next() + " = " + j.next();
        }
        str = (str == null ? "| otherwise = "
              : str + " | otherwise = ") + otherwise;
        return str;
    }
}
