package jpel.resolver;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import jpel.language.Environment;
import jpel.util.Debugger;
import jpel.util.dataholder.DataHolder;
import jpel.util.dataholder.DataHolderBuilder;
import jpel.util.dataholder.DataHolderException;
import jpel.util.dataholder.DataHolderExtractor;
import jpel.util.dataholder.DataHolderFactory;
import jpel.util.dataholder.DataHolderFactoryException;

/**
 * Politica que verifica periodicamente se as fontes de dados que geraram determinada
 * parametrizao foram modificadas, chamando os listeners sempre que isto acontece.
 */
public class PolicyActivateOnChange extends PolicyImpl implements Runnable {

    private Thread t;
    private long period = -1;

    private List configurations;
    private DataHolderFactory holderFactory;
    private DataHolderExtractor holderExtractor;
    private List resources;

    public PolicyActivateOnChange() throws PolicyException {
        try {
            configurations = Collections.synchronizedList(new LinkedList());
            holderFactory = DataHolderBuilder.getDataHolderFactory();
            holderExtractor = DataHolderBuilder.lookupDataHolderExtractor(
                              Environment.class.getName());
            resources = Collections.synchronizedList(new LinkedList());
        }
        catch(ClassNotFoundException exc) {
            throw new PolicyException(exc.getMessage(), exc);
        }
        catch(IllegalAccessException exc) {
            throw new PolicyException(exc.getMessage(), exc);
        }
        catch(InstantiationException exc) {
            throw new PolicyException(exc.getMessage(), exc);
        }
    }

    public long getPeriod() {
        return period;
    }

    public synchronized void setPeriod(long period) {
        this.period = period;
        notifyAll();
    }

    public void addConfiguration(Configuration conf) throws PolicyException {
        try {
            synchronized(configurations) {
                configurations.add(conf);
                List list = new LinkedList();
                DataHolder source = holderFactory.produce(conf.getSource());
                List dependecy = holderExtractor.extract(source);
                Iterator iter = dependecy.iterator();
                while(iter.hasNext()) {
                    DataHolder holder = (DataHolder) iter.next();
                    list.add(new TimeStamp(holder, holder.lastModified()));
                }
                resources.add(list);
            }
        }
        catch(ConfigurationException exc) {
            throw new PolicyException(exc.getMessage(), exc);
        }
        catch(DataHolderException exc) {
            throw new PolicyException(exc.getMessage(), exc);
        }
        catch(DataHolderFactoryException exc) {
            throw new PolicyException(exc.getMessage(), exc);
        }
    }

    public void removeConfiguration(Configuration conf) throws
        PolicyException {
        synchronized(configurations) {
            int index = configurations.indexOf(conf);
            configurations.remove(index);
            resources.remove(index);
        }
    }

    public void start() {
        if(t == null) {
            t = new Thread(this);
            t.start();
            Debugger.println("ActivateOnChange", "start", "Thread started.");
        }
        else {
            Debugger.println("ActivateOnChange", "start",
                "Thread already running.");
        }
    }

    public synchronized void run() {
        boolean ok = true;
        if(period < 0) {
            try {
                wait();
            }
            catch(InterruptedException ex) {
                ex.printStackTrace();
            }
        }
        while(ok) {
            try {
                checkAll();
            }
            catch(PolicyException exc) {
            //exc.printStackTrace();
            }
            try {
                Thread.sleep(period);
            }
            catch(InterruptedException ex) {
                ok = false;
            }
        }
    }

    public void checkAll() throws PolicyException {
        Iterator iter = configurations.iterator();
        Iterator iterHolder = resources.iterator();
        while(iter.hasNext() && iterHolder.hasNext()) {
            Configuration conf = (Configuration) iter.next();
            List list = (List) iterHolder.next();
            Iterator in = list.iterator();
            tag:while(in.hasNext()) {
                TimeStamp stamp = (TimeStamp) in.next();
                try {
                    Debugger.println("OnChange", "checkAll",
                        "Checking " + stamp.holder);
                    long time = stamp.holder.lastModified();
                    if(time != stamp.time) {
                        firePolicyActivated(new PolicyEvent(conf));
                        this.removeConfiguration(conf);
                        this.addConfiguration(conf);
                        break tag;
                    }
                }
                catch(DataHolderException exc) {
                    Debugger.println("OnChange", "checkAll",
                        "Unable to check " + stamp.holder);
                }
            }
        }
        Debugger.println("OnChange", "checkAll", "Checked.");
    }

    private class TimeStamp {
        public DataHolder holder;
        public long time;
        public TimeStamp(DataHolder holder, long time) {
            this.holder = holder;
            this.time = time;
        }
    }
}