package jpel.tree;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;

/**
 * Representa um n de uma rvore n-ria.
 */
public interface Node {

    /**
     * Retorna o objeto associado ao n dado.
     * @return O objeto associdado.
     */
    Object getValue();

    /**
     * Ajusta o dado associado ao n.
     * @param value O dado do n.
     * @throws NodeException Quando no  possvel ajustar o dado.
     */
    void setValue(Object value) throws NodeException;

    /**
     * Retorna o pai do n.
         * @return Um objeto do tipo Node imediatamente acima deste, ou null, caso o
     * n no tenha pai.
     */
    Node getParent();

    /**
     * Ajusta o pai do n.
     * @param parent Novo pai do n.
     * @throws NodeException Quando no  possvel ajustar o pai.
     */
    void setParent(Node parent) throws NodeException;

    /**
     * Retorna o n raiz relativo a este n.
         * @return O prprio n, quando ele  a raiz, ou o n mais alta na hierarquia
     * ao qual este n pertence.
     */
    Node getRoot();

    /**
         * Retorna indicao se o n dado  raiz ou no. Um n  considerado raiz se
     * o seu pai  nulo.
     * @return true, se o n  raiz, false, caso contrrio.
     */
    boolean isRoot();

    /**
     * Indica se o n est vazio ou no. Basicamente  uma abreviao para a
     * pergunta: size() != 0 ?
     * @return true Caso haja sub ns, else, caso tenha.
     */
    boolean isFolder();

    /**
     * Indica se o n est vazio ou no. Basicamente  uma abreviao para a
     * pergunta: size() == 0 ?
     * @return true Caso no haja sub ns, false, caso tenha.
     */
    boolean isLeaf();

    /**
     * Retorna o caminho do n at a raiz da rvore. O primeiro n da lista
     *  a raiz e o ltimo n  o prprio n. Para acessar o caminho  s
     * utilizar os ndices.
     * @return O caminho para acessar o n atravs da raiz. Se o n chamado for
     * raiz, o caminho retornado  vazio.
     */
    Node getPath();

    /**
     * Remove todos os sub-ns.
     */
    void clear();

    /**
     * Insere ns no final da lista.
     * @param node O n a ser inserido.
     * @throws InsertNodeException Caso haja problemas com a insero.
     */
    void insert(Node node) throws InsertNodeException;

    /**
     * Insere ns em determinadas posies da lista.
     * @param node O n a ser inserido.
     * @param index Posio onde deve ser inserido o n.
     * @throws InsertNodeException Caso haja problemas com a insero.
     */
    void insert(Node node, int index) throws InsertNodeException;

    /**
     * Remove o n do seu respectivo pai. Caso o n seja uma raiz, nada ser
     * feito.
     * @throws RemoveNodeException Quando no  possvel remover o n.
     */
    void remove() throws RemoveNodeException;

    /**
     * Remove um n da lista.
     * @param index O ndice do n a ser removido.
     * @throws RemoveNodeException Caso haja problemas com a remoo.
     */
    void remove(int index) throws RemoveNodeException;

    /**
     * Remove um n da lista.
     * @param node O n a ser removido.
     * @throws RemoveNodeException Caso haja problemas com a remoo.
     */
    void remove(Node node) throws RemoveNodeException;

    /**
     * No pai, move/desloca este n para cima em sua lista de sub-ns.
     * @throws MoveNodeException Quando h problemas para mover.
     */
    void moveUp() throws MoveNodeException;

    /**
     * No pai, move/desloca este n para baixo em sua lista de sub-ns.
     * @throws MoveNodeException Quando h problemas para mover.
     */
    void moveDown() throws MoveNodeException;

    /**
     * Promove um n para um nvel acima de onde ele est. Caso o n j esteja
     * sob a raiz nada acontece.
     * @throws MoveNodeException Quando no  possvel promover o n.
     */
    void promote() throws MoveNodeException;

    /**
     * Indica se o n dado existe.
     * @param node N a ser procurado.
     * @return true, se o n existe como filho do n corrente, false, caso
     * contrrio.
     */
    boolean contains(Node node);

    /**
     * O ndice deste n em relao  lista do seu pai.
         * @return Um inteiro entre 0 e o nmero de sub-ns do pai, caso ele exista.
     * @throws SelectionNodeException Quando o n no tem pai.
     */
    int indexOf() throws SelectionNodeException;

    /**
     * Indica o positionamento de um dentro da lista.
     * @param node O n do qual se deseja obter o ndice.
         * @return Um inteiro no intervalo de 0 a size(), que indica onde est o n.
     * @throws SelectionNodeException Quando o elemento no est presente na
     * lista.
     */
    int indexOf(Node node) throws SelectionNodeException;

    /**
     * Retorna o primeiro n na lista da lista.
     * @return O primeiro n da lista.
     * @throws SelectionNodeException Caso no seja possvel selecionar tal n.
     */
    Node first() throws SelectionNodeException;

    /**
     * Retorna o ltimo n na lista da lista.
     * @return O ltimo n da lista.
     * @throws SelectionNodeException Caso no seja possvel selecionar tal n.
     */
    Node last() throws SelectionNodeException;

    /**
     * Retorna o N cujo indice foi dado.
         * @param index ndice do n que se deseja selecionar. Este valor deve variar
         * entre 0 e size(), caso o valor represente um ndice invlido a execeo 
     * lanada.
     * @return O n cujo ndice foi dado.
     * @throws SelectionNodeException Caso no seja possvel selecionar tal n.
     */
    Node nodeAt(int index) throws SelectionNodeException;

    /**
     * Retorna a enumerao dos ns da lista.
     * @return Um iterator sobre os ns da lista.
     */
    Iterator children();

    /**
     * Retorna o nmero de itens no folder.
     * @return O nmero de ns.
     */
    int size();

    /**
     * Retorna uma cpia da estrutura do n, como os mesmos dados agregados que
     * o n original. Mundanas na estrutura "copiada" no se refletem no n
     * original, mas mudanas nso objetos "pendurados" sim.
     * @return Uma cpia com a estrutura do n dado.
     * @throws NodeException Quando no  possvel fazer a cpia.
     */
    Node mirror() throws NodeException;
}
