package jpel.tree;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Implementao padro do n. Um n pode ser um folder de sub-ns ou ainda
 * seus filhos representarem um caminho da raiz at o pai (veja getPath()).
 */
public class NodeImpl implements Node {

    private Object value;
    private Node parent;
    private List list = new LinkedList();

    public NodeImpl() throws NodeException {
        this(null, null);
    }

    public NodeImpl(Object value) throws NodeException {
        this(value, null);
    }

    public NodeImpl(Object value, Node parent) throws NodeException {
        setValue(value);
        setParent(parent);
    }

    public Object getValue() {
        return value;
    }

    public void setValue(Object value) throws NodeException {
        this.value = value;
    }

    public Node getParent() {
        return parent;
    }

    public void setParent(Node parent) throws NodeException {
        this.parent = parent;
    }

    public Node getRoot() {
        return isRoot() ? this : getParent().getRoot();
    }

    public boolean isRoot() {
        return(parent == null);
    }

    public boolean isFolder() {
        return(size() != 0);
    }

    public boolean isLeaf() {
        return(size() == 0);
    }

    public Node getPath() {
        Node path = null;
        try {
            path = new NodeImpl();
            path.insert(new NodeImpl(getValue()));
            Node aux = getParent();
            while(aux != null) {
                path.insert(new NodeImpl(aux.getValue()), 0);
                aux = aux.getParent();
            }
        }
        catch(InsertNodeException exc) {
            exc.printStackTrace();
        }
        catch(NodeException exc) {
            exc.printStackTrace();
        }
        return path;
    }

    public void clear() {
        list.clear();
    }

    public void insert(Node node) throws InsertNodeException {
        /** repetio  restrio de neggio no de dados.
                 if( contains(node) ) {
            throw new InsertNodeException("Could not insert node '"+node+"'. Node already exists.");
                 }
         */
        try {
            node.setParent(this);
        }
        catch(NodeException exc) {
            throw new InsertNodeException(exc.getMessage(), exc);
        }
        list.add(node);
    }

    public void insert(Node node, int index) throws InsertNodeException {
        /** repetio  restrio de neggio no de dados.
                 if( contains(node) ) {
            throw new InsertNodeException("Could not insert node '"+node+"'. Node already exists.");
                 }
         */
        if(!indexOk(index)) {
            throw new InsertNodeException("Could not insert node '" + node
                + "' at " + index + ", out of node range [0," + size() + "]");
        }
        try {
            node.setParent(this);
        }
        catch(NodeException exc) {
            throw new InsertNodeException(exc.getMessage(), exc);
        }
        list.add(index, node);
    }

    public void remove() throws RemoveNodeException {
        if(parent != null) {
            parent.remove(this);
        }
    }

    public void remove(int index) throws RemoveNodeException {
        if(!indexOk(index)) {
            throw new RemoveNodeException("Could not remove node '" + index
                + "', out of node range [0," + size() + "]");
        }
        try {
            remove(nodeAt(index));
        }
        catch(SelectionNodeException exc) {
            throw new RemoveNodeException(exc.getMessage(), exc);
        }
    }

    public void remove(Node node) throws RemoveNodeException {
        if(!contains(node)) {
            throw new RemoveNodeException("Could not remove node '" + node
                + "'. Node does not exist.");
        }
        try {
            node.setParent(null);
        }
        catch(NodeException exc) {
            throw new RemoveNodeException(exc.getMessage(), exc);
        }
        list.remove(node);
    }

    public void moveUp() throws MoveNodeException {
        if(parent != null) {
            try {
                int index = indexOf();
                if(index > 0) {
                    Node p = parent;
                    remove();
                    p.insert(this, index - 1);
                }
            }
            catch(SelectionNodeException exc) {
                throw new MoveNodeException(exc.getMessage(), exc);
            }
            catch(RemoveNodeException exc) {
                throw new MoveNodeException(exc.getMessage(), exc);
            }
            catch(InsertNodeException exc) {
                throw new MoveNodeException(exc.getMessage(), exc);
            }
        }
    }

    public void moveDown() throws MoveNodeException {
        if(parent != null) {
            try {
                int index = indexOf();
                if(index < parent.size() - 1) {
                    Node p = parent;
                    remove();
                    p.insert(this, index + 1);
                }
            }
            catch(SelectionNodeException exc) {
                throw new MoveNodeException(exc.getMessage(), exc);
            }
            catch(RemoveNodeException exc) {
                throw new MoveNodeException(exc.getMessage(), exc);
            }
            catch(InsertNodeException exc) {
                throw new MoveNodeException(exc.getMessage(), exc);
            }
        }
    }

    public void promote() throws MoveNodeException {
        if(parent != null) {
            try {
                Node p = parent.getParent();
                if(p != null) {
                    int index = parent.indexOf();
                    remove();
                    p.insert(this, index + 1);
                }
            }
            catch(RemoveNodeException exc) {
                throw new MoveNodeException(exc.getMessage(), exc);
            }
            catch(InsertNodeException exc) {
                throw new MoveNodeException(exc.getMessage(), exc);
            }
            catch(SelectionNodeException exc) {
                throw new MoveNodeException(exc.getMessage(), exc);
            }
        }
    }

    public boolean contains(Node node) {
        return list.contains(node);
    }

    public int indexOf() throws SelectionNodeException {
        if(parent == null) {
            throw new SelectionNodeException("Could not find node '"
                + this +"', parent is null.");
        }
        return parent.indexOf(this);
    }

    public int indexOf(Node node) throws SelectionNodeException {
        if(!contains(node)) {
            throw new SelectionNodeException("Could not find node '" + node
                + "' index.");
        }
        return list.indexOf(node);
    }

    public Node first() throws SelectionNodeException {
        if(isLeaf()) {
            throw new SelectionNodeException("Empty node!");
        }
        return nodeAt(0);
    }

    public Node last() throws SelectionNodeException {
        if(isLeaf()) {
            throw new SelectionNodeException("Empty node!");
        }
        return nodeAt(size() - 1);
    }

    public Node nodeAt(int index) throws SelectionNodeException {
        if(!indexOk(index)) {
            throw new SelectionNodeException("Could not select node '" + index
                + "', out of node range [0," + size() + "]");
        }
        return(Node) list.get(index);
    }

    public Iterator children() {
        return list.iterator();
    }

    public int size() {
        return list.size();
    }

    private boolean indexOk(int index) {
        return(index >= 0 && index <= size());
    }

    public String toString() {
        return(size() > 0 ? "Folder"
            : "Node") + ":" + String.valueOf(value) + "." + list;
    }

    public boolean equals(Object obj) {
        boolean result = false;
        if(obj instanceof Node) {
            Object objValue = ((Node) obj).getValue();
            if(value != null) {
                result = value.equals(objValue);
            }
            else {
                result = (objValue == null);
            }
        }
        return result;
    }

    public Node mirror() throws NodeException {
        Node result = new NodeImpl(value);
        if(isFolder()) {
            Iterator iter = children();
            while(iter.hasNext()) {
                Node child = (Node) iter.next();
                result.insert(child.mirror());
            }
        }
        return result;
    }
}
