package jpel.util.dataholder;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author
 * @version 1.0
 */

import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.io.Writer;

/**
 * Indica uma fonte de recursos para a linguagem.
 */
public interface DataHolder {

    /**
     * Indica uma fonte do tipo stream binrio.
     */
    int BINARYSTREAM = 0;
    /**
     * Indica uma fonte do tipo stream de caracteres.
     */
    int CHARACTERSTREAM = BINARYSTREAM + 1;
    /**
     * Indica uma fonte do tipo java.io.File.
     */
    int FILE = CHARACTERSTREAM + 1;
    /**
     * Indica uma fonte do tipo java.net.URL.
     */
    int URL = FILE + 1;
    /**
     * Indica uma fonte do tipo qualquer.
     */
    int OTHER = URL + 1;

    /**
     * Valor do lastModified quando no  possvel determinar o valor exato.
     */
    long DATE_UNKNOWN = -1;

    /**
     * Ajusta o tipo do manipulador de recursos.
     * @param type Tipo da fonte.
     */
    void setType(int type);

    /**
     * Indica o tipo do manipulador de recursos.
     * @return O inteiro correspondente ao tipo de fonte. Pode ser INPUTSTREAM,
     * READER, FILE, URL ou OTHER.
     */
    int getType();

    /**
     * Ajusta o objeto usado como referencia pela pelo manipulador de dados.
     * @param reference Referncia do manipulador de dados.
     */
    void setReference(Object reference);

    /**
     * Retorna o objeto java associado a fonte de recursos. No caso do tipo
         * FILE pode ser um java.io.File, no caso do tipo URL um objeto java do tipo
     * java.net.URL.
     * @return O objeto Java utilizado.
     */
    Object getReference();

    /**
     * Quando h a utilizao de recursos com relao entre si pode haver a necessidade
     * de resoluo de nomes relativos. Por exemplo, a relao entre arquivos.
     * @param relative O recurso ao qual se deseja resolver para o universo do recurso
     * atual.
     * <br> Por exemplo: Se a fonte atual de recursos(a) fosse o arquivo
     * examples/full/name.cfg e houvesse outra fonte de recursos(b) com o nome
         * ../resource.cfg, aps a aplicao do a.resolve(b) haveria um fonte de recursos
     * com uma referncia para o arquivo examples/resource.cfg.
     * @return A nova fonte de recursos resolvida.
     * @throws DataHolderException Quando no  possvel construir esta nova
     * fonte de recursos.
     */
    DataHolder resolve(DataHolder relative) throws DataHolderException;

    /**
     * Converte uma referencia absoluta em uma referncia relativa a este manipulador.
     * @param absolute Referncia absoluta a ser convertida.
     * <br> Por exemplo: Se a fonte atual de recursos(a) fosse o arquivo
         * c:/examples/full/name.cfg e houvesse outra fonte de recursos(b) com o nome
         * c:/subdir/resource.cfg, aps a aplicao do a.relative(b) haveria um fonte
     * de recursos com uma referencia para o arquivo ../../subdir/resource.cfg.
     * @return A fonte de dados relativa.
     * @throws DataHolderException Quando no  possvel construir esta nova
     * fonte de recursos.
     */
    DataHolder relative(DataHolder absolute) throws DataHolderException;

    /**
     * Retorna data da ltima modificao feita nos dados.
     * @return O tempo em milisegundos da ultima modificao.
     * @throws DataHolderException Quando no foi possvel descrever a data.
     */
    long lastModified() throws DataHolderException;

    /**
     * Retorna o buffer binrio associado a fonte de recursos.
     * @return Um buffer p/a leitura do recurso.
     * @throws DataHolderException Quando h erro no fornecimento deste buffer.
     */
    InputStream getInputStream() throws DataHolderException;

    /**
     * Retorna o buffer binrio associado a fonte de recursos.
     * @return Um buffer p/a escrita no recurso.
     * @throws DataHolderException Quando h erro no fornecimento deste buffer.
     */
    OutputStream getOutputStream() throws DataHolderException;

    /**
     * Retorna um buffer de caracteres associado a fonte de recursos.
     * @return Um buffer p/leitura do recurso.
     * @throws DataHolderException Quando h erro no fornecimento deste buffer.
     */
    Reader getReader() throws DataHolderException;

    /**
     * Retorna um buffer de caracteres associado a fonte de recursos.
     * @return Um buffer p/escrita no recurso.
     * @throws DataHolderException Quando h erro no fornecimento deste buffer.
     */
    Writer getWriter() throws DataHolderException;

    /**
     * Libera todos os recursos utilizados pela fonte de recursos.
     * @throws DataHolderException Quando h falha na liberao de recursos.
     */
    void close() throws DataHolderException;
}